package com.iona.fuse.demo.logisticx.derby;

/**
 * 
 * Copyright  2007-2008 IONA Technologies PLC.
 *           All Rights Reserved.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * http://www.apache.org/licenses/LICENSE-2.0
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 **/

/**
 * The Derby external service - this class launches the 
 * Derby Network Server Control on localhost and on port
 * 1527 by default. 
 * 
 */

import java.io.IOException;
import java.io.OutputStream;
import java.io.PrintWriter;
 
import org.apache.derby.drda.NetworkServerControl;
import org.springframework.beans.factory.DisposableBean;
import org.springframework.beans.factory.FactoryBean;
import org.springframework.beans.factory.InitializingBean;
 
public class NetworkServerBean implements FactoryBean, InitializingBean, DisposableBean {
 
   NetworkServerControl serverControl;
 
   /*
    * Gets an instance of the Derby Network Server class
    */
   public Class getObjectType() {
       return org.apache.derby.drda.NetworkServerControl.class;
   }
 
   public boolean isSingleton() {
       return false;
   } 
 
   public Object getObject() {
       return serverControl;
   }
 
   /*
    * Starts the Derby Network Server on port 1527
    */
   public void afterPropertiesSet() throws Exception {
 
       PrintWriter printWriter = makePrintWriter(System.out);     
       serverControl = new NetworkServerControl();
       serverControl.start(printWriter);
 
   } 
 
  /*
   * Shutdown the Derby Network Server
   */
  public void destroy() throws Exception {
 
       System.out.println("Destroying Bean");
 
       try {
           serverControl.shutdown();
       } catch (IOException ex) {
               System.err.println("WARN: Could not stop ServerControl"
                       + " trying to shutdown Derby Network Server");
       }
   }
 
   
   private static PrintWriter makePrintWriter(OutputStream out) {
           if (out != null) {
                   return new PrintWriter(out, true);
           } else {
                   return null;
           }
 
   }
 
 
 
}

