/**
 * 
 * Copyright  2007-2008 IONA Technologies PLC.
 *           All Rights Reserved.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * http://www.apache.org/licenses/LICENSE-2.0
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 **/

/*
 * Copyright 2007 Google Inc.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package com.iona.fuse.demo.logisticx.web.customer.client.customer;

import com.google.gwt.user.client.ui.Composite;
import com.google.gwt.user.client.ui.FlexTable;
import com.iona.fuse.demo.logisticx.web.customer.client.customer.model.OrderStatus;

public class MessageList extends Composite {

    private static final int VISIBLE_MESSAGE_COUNT = 10;
    private FlexTable table = new FlexTable();
    private int messageCount = 0;

    public MessageList() {
        table.setCellSpacing(0);
        table.setCellPadding(0);
        table.setWidth("100%");
        table.setHeight("100%");

        initWidget(table);
        // Associate the messages-List CSS style defined in
        // public/Logisticx.css with this gwt UI Object.
        setStyleName("messages-List");

        initTable();
        try {
            // Add a handler to receive update messages regarding our order
            // status from a jms queue.
            addOrderStatusHandler();
        } catch (Throwable e) {
            com.google.gwt.user.client.Window.alert(
                "Error adding order status handler :" + e);
        }
    }

    private void initTable() {
        table.setText(0, 0, "Customer PO#");
        table.setText(0, 1, "Order ID");
        table.setText(0, 2, "Comments");
        table.getRowFormatter().setStyleName(0, "messages-ListHeader");

        for (int i = 0; i < VISIBLE_MESSAGE_COUNT; ++i) {
            table.setText(i + 1, 0, "");
            table.setText(i + 1, 1, "");
            table.setText(i + 1, 2, "");
            table.getCellFormatter().setWordWrap(i + 1, 0, false);
            table.getCellFormatter().setWordWrap(i + 1, 1, false);
            table.getCellFormatter().setWordWrap(i + 1, 2, false);
            table.getFlexCellFormatter().setColSpan(i + 1, 2, 2);
        }
    }

   /**
     *  A jnsi handler to receive message updates through activemq ajax.
     */
    public native void receiveOrderStatusMessage(Object elem) /*-{        
        try{
            if (elem.nodeName != null) {
                var customerPoNumber = "";
                var orderId = "";
                var comments = "";
        
                var orderStatusItem = elem.firstChild;
                while (orderStatusItem != null) {
                    if (orderStatusItem.nodeName == "order") {                    
                        var orderItem = orderStatusItem.firstChild;
                        while (orderItem != null) {                    
                            if (orderItem.nodeName == "customerPoNumber") {
                                var customerText = orderItem.firstChild;                           
                                customerPoNumber = customerText.nodeValue;
                            }
                            if (orderItem.nodeName == "orderId") {
                                var orderIdText = orderItem.firstChild;                           
                                orderId = orderIdText.nodeValue;
                            }
                            orderItem = orderItem.nextSibling;
                        }
                    }
                    if (orderStatusItem.nodeName == "comments") {
                        var commentsText = orderStatusItem.firstChild;            
                        comments = commentsText.nodeValue;
                    }
                    orderStatusItem = orderStatusItem.nextSibling;                    
                }
                if (customerPoNumber && orderId && comments) {
                    @com.iona.fuse.demo.logisticx.web.customer.client.Logisticx::updateStatusMessage(Ljava/lang/String;Ljava/lang/String;Ljava/lang/String;)(customerPoNumber, orderId, comments);
                }
            } 
        } catch (e) {
            alert("Cannot propagate values from queue message :" + e);
        }
    }-*/;
    

   /**
     * Register our receiveOrderStatusMessage handler with activemq.
     */
    private native void addOrderStatusHandler() /*-{

        try {
        var myDestination = "channel://Customer.Orders.Outbound";
       
            $wnd.amq.addListener("orderStatusListener", myDestination, 
              this.@com.iona.fuse.demo.logisticx.web.customer.client.customer.MessageList::receiveOrderStatusMessage(Ljava/lang/Object;));
        } catch (e) {
            alert("Error adding asynchronous order status handler :" + e);
        }
    }-*/;

    public void updateMessage(String customerPoNumber, String orderId,
        String comments) {
        for (int i = 1; i < table.getRowCount(); i++) {
            if (table.getText(i, 0).equals(customerPoNumber) 
                && table.getText(i, 1).equals(orderId)) {
                table.setText(i, 2, comments);
                break;
            }
        }
    }

    public void addMessage(OrderStatus message) {
        messageCount++;
        table.insertRow(1);
        table.setText(1, 0, message.getOrder().getCustomerPoNumber());
        table.setText(1, 1, message.getOrder().getOrderId().toString());
        table.setText(1, 2, message.getComments());
        if (table.getRowCount() > VISIBLE_MESSAGE_COUNT + 1) {
            table.removeRow(VISIBLE_MESSAGE_COUNT + 1);
        }
        //table.setText(messageCount, 0, message.getOrder().getCustomerPoNumber());
        //table.setText(messageCount, 1, message.getOrder().getOrderId().toString());
        //table.setText(messageCount, 2, message.getComments());
    }
}
