/**
 * 
 * Copyright  2007-2008 IONA Technologies PLC.
 *           All Rights Reserved.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * http://www.apache.org/licenses/LICENSE-2.0
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 **/
/**
 *
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.iona.fuse.demo.logisticx.web.customer.client.operations;

import java.util.HashMap;
import java.util.Map;

import com.google.gwt.user.client.rpc.IsSerializable;

/**
 * @version $Revision: 61221 $
 */
public class MBeanName implements IsSerializable {
    private String domain;
    private String canonicalName;
    private Map keyPropertyList;
    private String[] treePath;

    public MBeanName() {
    }

    public MBeanName(String domain, String canonicalName, String[] treePath, Map keyPropertyList) {
        this.domain = domain;
        this.canonicalName = canonicalName;
        this.treePath = treePath;
        this.keyPropertyList = new HashMap(keyPropertyList);
    }

    public String toString() {
        return canonicalName + keyPropertyList;
    }

    public String getDomain() {
        return domain;
    }

    public void setDomain(String domain) {
        this.domain = domain;
    }

    public String getCanonicalName() {
        return canonicalName;
    }

    public void setCanonicalName(String canonicalName) {
        this.canonicalName = canonicalName;
    }

    public Map getKeyPropertyList() {
        return keyPropertyList;
    }

    public void setKeyPropertyList(Map keyPropertyList) {
        this.keyPropertyList = keyPropertyList;
    }

    public String getType() {
        String answer = getProperty("type");
        if (isBlank(answer)) {
            return getProperty("Type");
        }
        return answer;
    }

    public String getName() {
        String[] path = getTreePath();
        int last = path.length - 1;
        if (last >= 0) {
            return path[last];
        }
        String answer = getProperty("name");
        if (isBlank(answer)) {
            answer = getProperty("Name");
            if (isBlank(answer)) {
                return getCanonicalName();
            }
        }
        return answer;
    }

    public String[] getTreePath() {
        return treePath;
    }

    public void setTreePath(String[] treePath) {
        this.treePath = treePath;
    }

    public static boolean isBlank(String text) {
        return text == null || text.trim().length() == 0;
    }

    public String getProperty(String name) {
        if (keyPropertyList != null) {
            return (String) keyPropertyList.get(name);
        }
        return null;
    }
}
