package com.iona.fuse.demo.logisticx.model;

/**
 * 
 * Copyright  2007-2008 IONA Technologies PLC.
 *           All Rights Reserved.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * http://www.apache.org/licenses/LICENSE-2.0
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 **/

import javax.persistence.OneToOne;
import javax.persistence.*;
import javax.persistence.Entity;

/**
 * An object to convey the status of the entire order. Does not yet handle partial fulfillment status.  
 * 
 * @author bsnyder
 *
 */
@Entity
public class OrderStatus {

    @Id
    @GeneratedValue
    private Long statusId;
  
    @OneToOne
    @JoinColumn(name="ORDER_ID")
    private Order order;
    
    private StatusCode statusCode; 
    
    private String comments;
    
    /**
     * ORDER_PLACED - the order has only been placed 
     * ORDER_FULFILLED - the order has been picked for customer 
     * ORDER_SHIPPED - the order is en route to the customer 
     * ORDER_DELIVERED - the order has already been delivered to the customer 
     * ORDER_POSTPONED - the order has been back ordered 
     * 
     * @author bsnyder
     *
     */
    public enum StatusCode {
        ORDER_PLACED, 
        ORDER_INSTOCK, 
        ORDER_AWAITINGSTOCK, 
        ORDER_SHIPPED, 
        ORDER_DELIVERED, 
        ORDER_POSTPONED
    };
    
    public OrderStatus() {
        setStatusCode(StatusCode.ORDER_PLACED);
    }

    
    public Long getStatusId() {
        return statusId;
    }

    public void setStatusId(Long statusId) {
        this.statusId = statusId;
    }
    
    /**
     * A constructor to set a different <tt>statusCode</tt>. E.g.: 
     * <p>
     * <code>
     *     OrderStatus status = new OrderStatus(OrderStatus.ORDER_POSTPONED);
     * </code>
     * 
     * @param statusCode
     */
    public OrderStatus(StatusCode statusCode) {
        this.statusCode = statusCode;
    }
    
    public Order getOrder() {
        return order;
    }

    public void setOrder(Order order) {
        this.order = order;
    }
    
    public StatusCode getStatusCode() {
        return statusCode;
    }
 
    public void setStatusCode(StatusCode statusCode) {
        this.statusCode = statusCode;
    }

    public String getComments() {
        return comments;
    }

    public void setComments(String comments) {
        this.comments = comments;
    }
    
    /**
     * Used to construct a jms order status text message. Doesn't
     * include all information, just because this is all we want for now.
     */
    public String toXml() {
        String xml = "<orderStatus>" + order.toXml()
            + "<comments>" + comments + "</comments>"
            + "</orderStatus>";
        return xml;
    }
}
