package com.iona.fuse.demo.logisticx.service.order;

/**
 * 
 * Copyright  2007-2008 IONA Technologies PLC.
 *           All Rights Reserved.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * http://www.apache.org/licenses/LICENSE-2.0
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 **/

import javax.jbi.component.ComponentContext;
import javax.jbi.messaging.DeliveryChannel;
import javax.jbi.servicedesc.ServiceEndpoint;
import javax.jws.WebMethod;
import javax.jws.WebService;

import com.iona.fuse.demo.logisticx.model.Order;
import com.iona.fuse.demo.logisticx.model.OrderStatus;
import com.iona.fuse.demo.logisticx.service.order.support.OrderServiceSupport;

/**
 * The order web service implementation offered to Logisticx customers. Used to
 * place orders and checking orderStatus.
 *
 * @author bsnyder
 *
 */
@WebService(name = "OrderService",
            serviceName = "OrderServiceService",
            portName = "OrderServicePort", 
            targetNamespace = "http://logisticx.demo.fuse.iona.com/orderService/")
public class OrderServiceImpl implements OrderService {

    OrderServiceSupport support = new OrderServiceSupport();
    
    /** The JBI <a href="http://servicemix.apache.org/dist/servicemix-3.2.1/site/parent/core/servicemix-jbi/apidocs/javax/jbi/component/ComponentContext.html">
     * component context</a> to provide access to items such as the {@link DeliveryChannel}, {@link ServiceEndpoint}s, 
     * the endpoint registry and other important JBI 
     */ 
    private ComponentContext context;

    /**
     * Used to submit a single {@link com.iona.fuse.demo.logisticx.model.Order}
     */
    @WebMethod
    public OrderStatus putOrder(Order order) {
        // TODO: If we get a duplicate order, i.e., has a pre-existing OrderId,
        // we should probably throw an exception or return the existing
        // OrderStatus... the OrderProcessor won't like duplicate OrderIds.
        
        support.persist(order);
        OrderStatus status = new OrderStatus(OrderStatus.StatusCode.ORDER_PLACED);                
        status.setOrder(order);
        status.setComments("Thank you for your order!  Your order# is " + order.getOrderId());       
        support.persist(status);                                
        support.send(status);

        return status;
    }

    /**
     * Used to check the status of an order.
     */
    @WebMethod
    public OrderStatus getOrderStatus(long orderId) {        
        return support.retrieve(orderId);  
    }

    /** 
     * Used by the <a href="">servicemix-cxf-se</a> JBI component to inject the 
     * <a href="http://servicemix.apache.org/dist/servicemix-3.2.1/site/parent/core/servicemix-jbi/apidocs/javax/jbi/component/ComponentContext.html">
     * JBI component context</a>. 
     *  
     * @param context
     */
    public void setContext(ComponentContext context) {
        this.context = context;
        this.support.setContext(context);
    }

}
