package com.iona.fuse.demo.logisticx.stock.processor;

/**
 * 
 * Copyright  2007-2008 IONA Technologies PLC.
 *           All Rights Reserved.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * http://www.apache.org/licenses/LICENSE-2.0
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 **/

import javax.annotation.Resource;
import javax.jbi.component.ComponentContext;
import javax.jbi.messaging.DeliveryChannel;
import javax.jbi.messaging.ExchangeStatus;
import javax.jbi.messaging.InOnly;
import javax.jbi.messaging.InOut;
import javax.jbi.messaging.MessageExchange;
import javax.jbi.messaging.MessagingException;
import javax.jbi.messaging.NormalizedMessage;
import javax.jbi.servicedesc.ServiceEndpoint;
import javax.xml.namespace.QName;

import org.apache.servicemix.jbi.listener.MessageExchangeListener;

import org.apache.servicemix.jbi.jaxp.SourceTransformer;
import org.apache.servicemix.jbi.jaxp.StringSource;


public class StockProcessor implements MessageExchangeListener {

    @Resource private ComponentContext context;

    /** The namespace to use for calling the warehouse services **/
    private static final String WAREHOUSE_NAMESPACE = "http://logisticx.demo.fuse.iona.com/warehouseService/";

    /** The service name of the warehouse service **/
    private static final String WAREHOUSE_SERVICE_NAME = "WarehouseService";
    
    /** The name of the warehouse endpoint on the warehouse service **/
    private static final String WAREHOUSE_ENDPOINT_NAME = "WarehouseEndpoint";            
    @Resource            
    private DeliveryChannel channel;    
    
    public ComponentContext getContext() {
        return context;
    }
    
    public void onMessageExchange(MessageExchange exchange) throws MessagingException {
        if (exchange.getStatus() == ExchangeStatus.ACTIVE) {           
            boolean stockUpdated  = processStock(exchange); 
            exchange.setStatus(ExchangeStatus.DONE);
            channel.send(exchange);
        } 
    }

    /*
     * It will process the contents of the xml file that was
     * polled and send a request to the soap/http external
     * warehouse to update the stock inventory with the amount 
     * of new stock
     *
     */
    private boolean processStock(MessageExchange exchange) throws MessagingException 
    {
    	long orderId;
        InOut wareInOut = null;
    
      try {        
    	InOnly inOnly =  (InOnly)exchange;
        NormalizedMessage in = inOnly.getInMessage();            
        String content = (new SourceTransformer()).contentToString(in);     

        QName warehouseSrvName = new QName(WAREHOUSE_NAMESPACE, WAREHOUSE_SERVICE_NAME);
        ServiceEndpoint se = getContext().getEndpoint(warehouseSrvName, WAREHOUSE_ENDPOINT_NAME);
        wareInOut = channel.createExchangeFactory().createInOutExchange();                        
        wareInOut.setEndpoint(se);
        
        NormalizedMessage msg = wareInOut.createMessage();            
        msg.setContent(new StringSource(content));

        QName opName =  new QName("http://logisticx.demo.fuse.iona.com/warehouseService/", "updateStock");
        wareInOut.setOperation(opName);
        wareInOut.setMessage(msg, "in");
        wareInOut.setService(warehouseSrvName);       
        channel.sendSync(wareInOut); 
        wareInOut.setStatus(ExchangeStatus.DONE);
        channel.send(wareInOut);        
      } catch (Exception ex) {
          ex.printStackTrace();
      }     
      return true;
    }
    
}
