package com.iona.fuse.demo.logisticx.service.order;

/**
 * 
 * Copyright  2007-2008 IONA Technologies PLC.
 *           All Rights Reserved.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * http://www.apache.org/licenses/LICENSE-2.0
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 **/

import javax.xml.datatype.XMLGregorianCalendar;
import java.net.URL;
import java.util.logging.Logger;

import javax.xml.namespace.QName;
import javax.xml.ws.Endpoint;

import org.apache.servicemix.tck.SpringTestSupport;
import org.apache.xbean.spring.context.ClassPathXmlApplicationContext;
import org.springframework.context.support.AbstractXmlApplicationContext;

import org.apache.cxf.testutil.common.AbstractBusClientServerTestBase;
import org.apache.cxf.testutil.common.AbstractBusTestServerBase;

import org.apache.servicemix.jbi.container.JBIContainer;
import org.junit.BeforeClass;
import org.junit.Test;

//public class OrderServiceSystemTest extends SpringTestSupport {
public class OrderServiceSystemTest extends AbstractBusClientServerTestBase {

    static final Logger LOG = Logger.getLogger(OrderServiceSystemTest.class.getName());
    private JBIContainer container;

     public static class MyServer extends AbstractBusTestServerBase {

        // launches the order service standalone
        protected void run() {
            Object implementor = new OrderServiceImpl();
            String address = "http://localhost:8192/order-service";
            Endpoint.publish(address, implementor);

        }

        public static void main(String[] args) {
            try {
                MyServer s = new MyServer();
                s.start();
            } catch (Exception ex) {
                ex.printStackTrace();
                System.exit(-1);
            } finally {
                LOG.info("done!");
            }
        }
    }
    

/*     @BeforeClass
    public void setUp() throws Exception {
        container = new JBIContainer();
        container.setEmbedded(true);
        container.init();
        container.start();

    }*/

    /*
     * Uses some CXf TestUtilities to launch the OrderService
     */
    @BeforeClass
    public static void startServers() throws Exception {
        assertTrue("server did not launch correctly", launchServer(MyServer.class));
    }
    
    /*@AfterClass
    public void tearDown() throws Exception {
        container.shutDown();
    }*/

    @Test
    public void testPutOrder() throws Exception {
        
        QName serviceName = new QName("http://logisticx.demo.fuse.iona.com/orderService", "OrderServiceService");
        
        URL wsdlUrl = OrderServiceSystemTest.class.getClassLoader().getResource(
                "com/iona/fuse/demo/logisticx/service/order/orderService.wsdl");

        /*
        * Create a Service object for the service on which the 
        * consumer will invoke operations.
        */
        OrderServiceService orderService = new OrderServiceService(wsdlUrl, serviceName);

        /*
        * Adds an endpoint definition to the Service object 
        * using the addPort() method
        */
        QName portName = new QName("http://logisticx.demo.fuse.iona.com/orderService", 
                               "OrderServicePort");
        orderService.addPort(portName, "http://schemas.xmlsoap.org/soap/",
                        "http://localhost:8192/order-service");

        /*
        * Get a service proxy for the service         
        */
        OrderService client = orderService.getPort(portName, OrderService.class);        

        // Invokes the client operations.
        PutOrder putOrder = new PutOrder();
        Order newOrder = new Order();
        newOrder.setCustomerId(1234);
        newOrder.setCustomerPoNumber("testPO#1");
        putOrder.setOrder(newOrder);
                      
        PutOrderResponse response = client.putOrder(putOrder);
        OrderStatus status = response.getReturn();
        assertNotNull("OrderID is not null", status.getOrder().getOrderId());
   
        newOrder.setOrderId(status.getOrder().getOrderId()); 
        status = client.getOrderStatus(newOrder.getOrderId());
        assertEquals("ORDER_PLACED", status.getStatusCode().value());
        
    }

    /*@Override
    protected AbstractXmlApplicationContext createBeanFactory() {
        return new ClassPathXmlApplicationContext(
            "com/iona/fuse/demo/logisticx/service/order/xbean.xml");
    }*/


}
