package com.iona.fuse.demo.logisticx.service.warehouse;

/**
 * 
 * Copyright  2007-2008 IONA Technologies PLC.
 *           All Rights Reserved.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * http://www.apache.org/licenses/LICENSE-2.0
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 **/

import java.util.List;
import javax.jws.WebMethod;
import javax.jws.WebService;

import com.iona.fuse.demo.logisticx.model.LineItem;
import com.iona.fuse.demo.logisticx.model.StockItem;
import com.iona.fuse.demo.logisticx.model.Order;
import com.iona.fuse.demo.logisticx.model.OrderStatus;

/**
 * Base class implementation of the Warehouse Services.
 * @param order The order object being submitted by the customer
 * @return The status of the order 
 * 
 * @ORDER_INSTOCK, @ORDER_AWAITINGSTOCK
 * 
 */
public class WarehouseBaseImpl implements Warehouse {
    
    Order ord = null;    
    
    /** The Helper class to access database objects. **/
    private WarehouseHelper warehouseHelper = new WarehouseHelper();

    /**
     * Used to determine if items are in stock
     */
    @WebMethod
     public OrderStatus inStock(Order order) {
        OrderStatus stat = null;
        
        if (order != null) {
            System.out.println("Processing Stock Order");
            stat = order.getStatus();
            LineItem items = order.getLineItem();
            long id = items.getItemId();
            int qty = items.getQuantity();            
            int totalInStock = warehouseHelper.JpaRetrieveStockItem(id).getQuantity();
            System.out.println("Required = " + qty + " Total in stock = " + totalInStock);
            if (qty <= totalInStock) {
                warehouseHelper.JpaUpdateOrderStockItem(id, qty);
                stat.setStatusCode(OrderStatus.StatusCode.ORDER_INSTOCK);
            } else {
               stat.setStatusCode(OrderStatus.StatusCode.ORDER_AWAITINGSTOCK);
            }                                
        }
        return stat;
    }

    /*
     * Increases the amount of stock 
     * available for ordering.
     *
     */
    @WebMethod 
    public boolean updateStock(StockItem item) {
        
        System.out.println("Processing new Stock");
        if (item != null) {                   
            StockItem stockItem = warehouseHelper.JpaUpdateNewProductStockItem(item.getProductId(), item.getQuantity());
            if (stockItem != null) {
                System.out.println("OldStock = " + item.getQuantity() + " NewStock = " + stockItem.getQuantity());
                return true;
            }
        }
        return false;
    }


}
